import sys
import os
import os.path
import fnmatch
import shutil
import glob

#-------------------------------------------------------------------------------
# the main function
# cd tools
# python elxReplaceCopyrightNotice.py
def main() :

  # The path to the source files relative to this script
  srcdir = os.path.join( "..", "src" );

  # Get a list of all files
  patterns = ( '*.h', '*.hxx', '*.cxx', '*.cuh', '*.cu', '*.in' );
  matches = [];
  for pattern in patterns :
    for root, dirnames, filenames in os.walk( srcdir ):
      for filename in fnmatch.filter( filenames, pattern ) :
        matches.append( os.path.join( root, filename ) );

  # Read the old copyright notice from file
  noticeOld = "CopyrightNotice_BSD.txt";
  needle      = open( noticeOld ).read();
  #print( needle );

  # Read the new copyright notice from file
  noticeNew = "CopyrightNotice_Apache.txt";
  replacement = open( noticeNew ).read();
  #print( replacement );

  for match in matches :
    # Print the current file name
    print( match );
    #continue;

    # Read the current file
    inp1 = open( match, 'r' );
    fileAsString = inp1.read();
    inp1.close();

    # Replace the copyright notice
    inp2 = open( match, 'w' );
    inp2.write( fileAsString.replace( needle, replacement ) );
    inp2.close();

  # Exit
  return 0

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    sys.exit(main())



import sys
import os
import os.path
import subprocess
import shutil
import platform
import argparse


#-------------------------------------------------------------------------------
#
# This python script runs nightly dashboards for elastix.
#
# To circumvent all kinds of problems, the latest nightly dashboard scripts
# are checked out to a temporary directory, after which they are called.
# After running the dashboard these temporary scripts are deleted again.
#
# This python script assumes that svn, git and ctest can be found on the PATH.
#
def main():

  # Create an argument parser
  parser = argparse.ArgumentParser()

  # Argument to select the dashboard script (required)
  # e.g. -s elxDashboard_LKEB_win10-64_VS2015.cmake
  parser.add_argument( "-s", "--script",
    dest='dashboard_script', required=True,
    choices = [
      'elxDashboard_LKEB_win7-64_VS2010.cmake',
      'elxDashboard_LKEB_win10-64_VS2015.cmake',
      'elxDashboard_LKEB_linux64_gcc_Debug.cmake',
      'elxDashboard_LKEB_linux64_gcc_Release.cmake',
      'elxDashboard_LKEB_linux64_clang_Debug.cmake',
      'elxDashboard_LKEB_linux64_clang_Release.cmake',
      'elxDashboard_LKEB_macosx64_gcc.cmake',
      'elxDashboard_BIGR_linux64_gcc.cmake',
      'elxDashboard_BIGR_winXP32_VS2008.cmake' ],
    help='select the dashboard to run' );

  # Argument to select the compilation mode (optional)
  # e.g. -C Debug
  parser.add_argument( "-C", default='Release', dest='C',
    choices = [ 'Debug', 'Release' ],
    help='select the compilation mode' );

  # Argument to select the local working directory (required)
  # e.g. -d "D:/toolkits/elastix/nightly"
  # e.g. -d  "/home/marius/nightly-builds"
  parser.add_argument( "-d", "--dir",
    dest='local', required=True,
    help='set the local working directory' );

  # Argument to select the elastix remote (optional)
  # By default it points to https://github.com/SuperElastix/elastix
  # but this option can be used to select user-specific forks.
  # e.g. -r https://github.com/SuperElastix/elastix
  # e.g. -r https://github.com/mstaring/elastix
  #parser.add_argument( "-r", "--remote",
  #  default='https://github.com/SuperElastix/elastix', dest='remote_base',
  #  help='select elastix remote' );
  # This is currently hard-coded in elxDashboardCommon.cmake,
  # and cannot be an option here atm.
  remote_base = "https://github.com/SuperElastix/elastix";

  # Argument to select the elastix branch (optional)
  # e.g. -b performance_ITK4
  #parser.add_argument( "-b", "--branch",
  #  default='develop', dest='branch',
  #  help='select elastix branch' );
  # This is currently hard-coded in elxDashboardCommon.cmake,
  # and cannot be an option here atm.
  branch = "develop";

  options = parser.parse_args();

  # Define and open log file
  tmpname = options.dashboard_script.rstrip( '.cmake' ).lstrip( 'elxDashboard_' );
  log  = options.local + "/nightlybuild_" + tmpname + ".log";
  flog = open( log, "w" );

  # Check if required executables can be found
  for exe in [ 'svn', 'git', 'ctest' ] :
    # Since whutil.which() only exists from python 3.3 we use try
    try :
      if shutil.which( exe ) == None :
        message = "ERROR: unable to find executable: " + exe;
        print( message );
        flog.write( message );
        exit();
    except : pass;

  # Remote. Define the remote location of the dashboard scripts
  remote        = remote_base + "/branches/" + branch + "/Testing/Dashboard";
  remote_common = remote + "/elxDashboardCommon.cmake";
  remote_dash   = remote + "/" + options.dashboard_script;

  # Local. Define the local nightly directory on this machine
  tmpdir        = options.local  + "/elx_" + tmpname + "_tmp";
  local_common  = tmpdir + "/elxDashboardCommon.cmake";
  local_dash    = tmpdir + "/" + options.dashboard_script;

  # Checkout the latest dashboard scripts
  #
  # The following relies on the GitHub support of Subversion.
  # It allows us to svn export only the dashboard files, and not
  # the complete elastix repository.
  #
  # elastix.git can be approached as follows:
  # svn export https://github.com/SuperElastix/elastix/trunk/path/to/file
  # for the main branch 'master'. Branches can be approached like:
  # svn export https://github.com/SuperElastix/elastix/branches/develop/path/to/file

  # Create local temporary directory to checkout dashboard scripts to
  shutil.rmtree( tmpdir, True );
  os.mkdir( tmpdir );

  # Export both the common and the specific dashboard files.
  command = [ "svn", "export", "--non-interactive", "--trust-server-cert" ];
  command.extend( [ remote_common, local_common ] );
  flog.write( ' '.join( command ) ); flog.write( '\n' );
  subprocess.call( command, stdout = flog, stderr = subprocess.STDOUT );

  command = [ "svn", "export", "--non-interactive", "--trust-server-cert" ];
  command.extend( [ remote_dash, local_dash ] );
  flog.write( ' '.join( command ) ); flog.write( '\n' );
  subprocess.call( command, stdout = flog, stderr = subprocess.STDOUT );

  flog.write( "Dashboard scripts checked out\n" );

  # Run the nightly dashboard!
  #
  command = [ "ctest", "-C", options.C, "-S", local_dash, "-V" ];
  flog.write( ' '.join( command ) ); flog.write( '\n' );
  subprocess.call( command, stdout = flog, stderr = subprocess.STDOUT );

  # Close log file
  flog.close();

  # Clean up, silence warnings
  if os.path.exists( tmpdir ) :
    shutil.rmtree( tmpdir, True );
  if os.path.exists( tmpdir ) :
    os.system( 'rmdir /S /Q \"' + tmpdir + '\"' );

#-------------------------------------------------------------------------------
if __name__ == '__main__' :
  sys.exit(main())



import sys
import os
import os.path
import platform
import shutil
from optparse import OptionParser

# -------------------------------------------------------------------------------
# the main function
#
# This python script applies uncrustify (Code Beautifier) to the elastix code.
#
# Uncrustify web site: http://uncrustify.sourceforge.net/
# Compile and add uncrustify to your $PATH environment variable.
#
# The configuration file for uncrustify is called elastixUncrustify.cfg.
#
# Execute this script without any option to create a preview directory
# called '_beautiful_code' in the src\_beautiful_code.
# To apply uncrustify directly to the code use option '-a'.
# Including and excluding files and directories could be also indicated, see options.
#
# The script should be executed from the tools directory.
#
# \author Denis P. Shamonin and Marius Staring. Division of Image Processing,
# Department of Radiology, Leiden, The Netherlands
#
# \note This work was funded by the Netherlands Organisation for
# Scientific Research (NWO NRG-2010.02 and NWO 639.021.124).

def main():
    # usage, parse parameters
    usage = "usage: %prog [options] arg";
    parser = OptionParser(usage);

    # option to debug and verbose
    parser.add_option("-d", "--debug", action="store_true", dest="debug",
                      default=False, help="debug commands calls");
    parser.add_option("-v", "--verbose", action="store_true",
                      default=False, dest="verbose", help="verbose");
    parser.add_option("-q", "--quiet", action="store_true",
                      default=False, dest="quiet", help="quiet mode for uncrustify");

    # user defined uncrustify configuration file
    parser.add_option("-c", "--config", dest="config",
                      type="string", help="uncrustify configuration file");

    # options to control files. use -o output1 not -o c:\\temp
    parser.add_option("-o", "--output", dest="output_directory",
                      default="_beautiful_code", help="relative uncrustify output directory");

    # include regex, use syntax -i "value1 value2" NOT -i value1 value2
    parser.add_option("-i", "--include-regex", dest="include",
                      type="string", help="include files matching regular expression");

    # exclude regex, use syntax -e "value1 value2" NOT -e value1 value2
    parser.add_option("-e", "--exclude-regex", dest="exclude",
                      default="Common/KNN/ann_1.1/ Common/CUDA/",
                      help="exclude files matching regular expression");

    # apply directly to the code, has to be confirmed
    parser.add_option("-a", "--apply", action="store_true", default=False,
                      dest="apply", help="apply uncrustify directly to the svn files");

    (options, args) = parser.parse_args();

    # check if option -a is given, and confirm
    if options.apply == True:
        answer = query_yes_no("WARNING: Do you want to apply Uncrustify directly to the code?")
        if answer == False:
            return 0

    # get include and exclude options as lists
    include_list = None;
    exclude_list = None;
    if options.include != None:
        include_list = options.include.split(" ");
    if options.exclude != None:
        exclude_list = options.exclude.split(" ");

    # check for contradictory options
    if options.include != None and options.exclude != None:
        for include in include_list:
            for exclude in exclude_list:
                if include == exclude:
                    print( "ERROR: The contradictory options provided -i " + include + " -e " + exclude + "." );
                    return 1;

    # list of valid C++ extensions and top directories
    src_valid_cxx_extensions = set([".h", ".cpp", ".cxx", ".hxx", ".txx"]);  # not .in.h
    src_top_dirs = ["Common", "Components", "Core", "Testing"];

    # Define the current working directory and the directory containing this script
    current_dir = os.getcwd();
    current_dir = os.path.abspath(os.path.join(current_dir, '../src'));
    current_dir = current_dir.replace("/cygdrive/c/", "C:/");  # Cygwin support
    current_dir = current_dir.replace("/cygdrive/d/", "D:/");  # Cygwin support
    script_dir = os.path.dirname(os.path.realpath(__file__));
    script_dir = script_dir.replace("/cygdrive/c/", "C:/");  # Cygwin support
    script_dir = script_dir.replace("/cygdrive/d/", "D:/");  # Cygwin support

    # Find the uncrustify configuration file
    #
    if options.config == None:
        # If no configuration file is given via the options, then we look for a default
        # First we look in the directory of this script
        app_uncrustify_cfg_file = os.path.join(script_dir, "elastixUncrustify.cfg");
        # If it doesn't exist we look in the current working directory
        if not os.path.exists(app_uncrustify_cfg_file):
            app_uncrustify_cfg_file = os.path.join(current_dir, "elastixUncrustify.cfg");
        # If it doesn't exist we throw an error
        if not os.path.exists(app_uncrustify_cfg_file):
            print( "ERROR: cannot find the configuration file elastixUncrustify.cfg" );
            print( "  looked in: " + script_dir );
            print( "  looked in: " + current_dir );
            return 1;
    else:
        # Otherwise we use the user supplied configuration file
        if not os.path.exists(options.config):
            print( "ERROR: The configuration file '" + options.config + "' does not exist." );
            return 1;

        app_uncrustify_cfg_file = os.path.realpath(options.config);
        app_uncrustify_cfg_file = app_uncrustify_cfg_file.replace("/cygdrive/c/", "C:/");  # Cygwin support
        app_uncrustify_cfg_file = app_uncrustify_cfg_file.replace("/cygdrive/d/", "D:/");  # Cygwin support

    if options.verbose == True:
        print( "DEBUG: uncrustify configuration file: " + app_uncrustify_cfg_file );

    # uncrustify executable
    uncrustify_exe_name = "uncrustify";

    # get system type and define uncrustify executable
    if get_system_name() == 'windows':
        uncrustify_exe_name += ".exe"

    # check that uncrustify executable exists
    if which(uncrustify_exe_name) == None:
        print( "ERROR: The executable file '" + uncrustify_exe_name + "' does not exist in your PATH variable." );
        return 1;

    # create files list for uncrustify and place it in the output directory
    output_dir = os.path.join(current_dir, options.output_directory);
    if options.apply == False:
        create_dir(output_dir, options);

    # files list for uncrustify option -F:
    # -F FILE: read files to process from FILE, one filename per line
    app_uncrustify_files_list = "uncrustify_files.txt";
    if options.apply == True:
        app_uncrustify_files_list = os.path.join(current_dir, app_uncrustify_files_list);
    else:
        app_uncrustify_files_list = os.path.join(output_dir, app_uncrustify_files_list);

    # create files list, it has to be defined in uncrustify specific way
    filelist = None;

    if options.verbose == True:
        print( "DEBUG: Opening file " + app_uncrustify_files_list );
    filelist = open(app_uncrustify_files_list, "w");

    for dir in src_top_dirs:
        local_current_dir = os.path.join(current_dir, dir);
        for root, dirs, files in os.walk(local_current_dir):
            for f in files:
                fullpath = os.path.join(root, f);
                if file_valid(fullpath, include_list, exclude_list, src_valid_cxx_extensions):
                    rfullpath = fullpath.replace(current_dir, '');
                    rfullpath = rfullpath.lstrip('\\');
                    rfullpath = rfullpath.lstrip('/');  # Windows support
                    if options.debug == True:
                        print( "DEBUG: " + rfullpath );
                    else:
                        filelist.write(rfullpath);
                        filelist.write('\n');

    if options.verbose == True:
        print( "DEBUG: Closing file " + app_uncrustify_files_list );
    filelist.close();

    # define uncrustify arguments
    arg = "-c " + app_uncrustify_cfg_file;
    if options.apply == True:
        arg = arg + " --no-backup";
    else:
        arg = arg + " --prefix=" + output_dir;
    arg = arg + " -F " + app_uncrustify_files_list;
    if options.quiet == True:
        arg = arg + " -q";

    # execute uncrustify
    os.chdir(current_dir);
    call_program(uncrustify_exe_name, arg, options);

    # delete filelist if option apply is used
    if os.access(app_uncrustify_files_list, os.F_OK) == True and options.apply == True:
        if options.verbose == True:
            print( "DEBUG: Deleting file " + app_uncrustify_files_list );
        os.remove(app_uncrustify_files_list);

    # report
    print( "SUCCESS! Your code is beautiful!" );
    if not options.apply:
        print( "Check the directory " + output_dir );

    return 0


# -------------------------------------------------------------------------------
# get_system_name
def get_system_name():
    # store the architecture string (heavy performance drain)
    current_system = platform.system();
    current_system = current_system.lower();
    if current_system == 'windows':
        return 'windows';
    elif current_system == 'linux':
        return 'linux';
    elif current_system == 'darwin':
        return 'linux';
    elif 'cygwin' in current_system:
        return 'linux';
    else:
        return current_system;


# -------------------------------------------------------------------------------
# query_yes_no
def query_yes_no(question, default="yes"):
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False};
    if default == None:
        prompt = " [y/n] ";
    elif default == "yes":
        prompt = " [Y/n] ";
    elif default == "no":
        prompt = " [y/N] ";
    else:
        raise ValueError("invalid default answer: '%s'" % default);

    while True:
        sys.stdout.write(question + prompt);

        # First one works in python2, last one in python3
        try:
            choice = raw_input().lower();
        except NameError:
            choice = input().lower();

        if default is not None and choice == '':
            print("1")
            return valid[default];
        elif choice in valid:
            print("2")
            return valid[choice];
        else:
            print("3")
            sys.stdout.write("Please respond with 'yes' or 'no' " \
                             "(or 'y' or 'n').\n");


# -------------------------------------------------------------------------------
# which
# Can be replaced by shutil.which() after python 3.3.
def which(program):
    import os

    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None


# -------------------------------------------------------------------------------
# create directory
def create_dir(dir_name, options):
    if os.access(dir_name, os.F_OK) == False:
        cmd = "DEBUG: Creating directory '%s'" % dir_name
        if options.debug == True:
            print( cmd );
        else:
            if options.verbose == True:
                print( cmd );
            os.mkdir(dir_name);


# -------------------------------------------------------------------------------
# file_valid
def file_valid(file, include_list, exclude_list, elx_extensions):
    # check exclude
    if exclude_list != None:
        for exclude in exclude_list:
            if file.find(exclude) != -1:
                return False;

    # check within include
    if include_list != None:
        for include in include_list:
            if file.find(include) != -1:
                for extension in elx_extensions:
                    if file.endswith(extension):
                        return True;
    else:
        for extension in elx_extensions:
            if file.endswith(extension):
                return True;

    return False;


# -------------------------------------------------------------------------------
# call_program
def call_program(app_name, app_arg, options):
    cmd = app_name;
    if len(app_arg) > 0:
        cmd = cmd + " " + app_arg;

    if options.verbose and options.debug == False:
        print( "Calling " + cmd );

    if options.debug == True:
        print( "DEBUG: Calling " + cmd );
    else:
        os.system(cmd);

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    sys.exit(main())



import sys
import os
import os.path
import fnmatch
import shutil
import glob
import subprocess
import re

#-------------------------------------------------------------------------------
# the main function
def main() :

  # The path to the source files relative to this script
  srcdir = os.path.join( "..", "src" );

  # Get a list of all files
  patterns = ( 'CMakeLists.txt', '*.cmake' );
  matches = [];
  for pattern in patterns :
    for root, dirnames, filenames in os.walk( srcdir ):
      for filename in fnmatch.filter( filenames, pattern ) :
        matches.append( os.path.join( root, filename ) );
  #print( matches );

  # Create a list of cmake commands (in lower case)
  # cmake --help-command-list >
  commandlistfilename = "commandlist.txt";
  commandlist = open( commandlistfilename, "w" );
  p = subprocess.Popen(
    "cmake --help-command-list", stdout=commandlist, shell=True );
  (output, err) = p.communicate();
  commandlist.close( );
  # Remove first line would be nice ...

  # Read lower case commands in a list
  commands = open( commandlistfilename ).read().splitlines();

  for match in matches :
    # Print the current file name
    print( match );
    #continue;

    # Read the current file as a string
    inp1 = open( match, 'r' );
    fileAsString = inp1.read();
    inp1.close();

    # Replace all entries from the dictionary
    for i in commands :
      ii = r'\b' + re.escape( i ) + r'\b\s*\(';
      re_i = re.compile( ii, re.IGNORECASE );
      fileAsString = re_i.sub( i + r'(', fileAsString );

    # Replace endif(...) with endif()
    re_endif = re.compile( r'endif\(.*\)' );
    fileAsString = re_endif.sub( r'endif()', fileAsString );

    # Overwrite
    inp2 = open( match, 'w' );
    inp2.write( fileAsString );
    inp2.close();

  # Clean up
  if os.path.exists( commandlistfilename ) : os.remove( commandlistfilename );

  # Exit
  return 0

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    sys.exit(main())



import sys
import os
import os.path
import fnmatch
import shutil
import glob
import itertools
import subprocess

#-------------------------------------------------------------------------------
# the main function
def main() :

  # The path to the source files relative to this script
  srcdir = os.path.join( "..", "src" );
  doxdir = os.path.join( "..", "dox" );
  toolsdir = os.path.join( "..", "tools" );

  # Get a list of all files
  patterns = ( '*.h', '*.hxx', '*.cxx', '*.cuh', '*.cu', '*.cl', '*.in', '*.txt', '*.tex', '*.bib', '*.cmake', '*.mhd', '*.html', '*.css', '*.dox', '*.py' );
  matches = [];
  for pattern in patterns :
    for root, dirnames, filenames in itertools.chain( os.walk( srcdir ), os.walk( doxdir ), os.walk( toolsdir ) ) :
      for filename in fnmatch.filter( filenames, pattern ) :
        matches.append( os.path.join( root, filename ) );

  for filename in matches :
    # Print the current file name
    print( filename );

    # Set the svn property
    #svn propset svn:eol-style native filename
    p = subprocess.Popen(
      "svn propset svn:eol-style native " + filename, stdout=subprocess.PIPE, shell=True );
    (output, err) = p.communicate();

  # Exit
  return 0

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    sys.exit(main())



import sys
import os
import os.path
from optparse import OptionParser

#-------------------------------------------------------------------------------
# the main function
def main():
    # usage, parse parameters
    usage = "usage: %prog [options] arg"
    parser = OptionParser( usage )

    # option to debug and verbose
    parser.add_option( "-v", "--verbose",
        action="store_true", dest="verbose" )

    # options to control files
    parser.add_option( "-d", "--directory", dest="directory", help="elastix output directory" )
    parser.add_option( "-b", "--baseline", dest="baseline", help="baseline string" )

    (options, args) = parser.parse_args()

    # loop in directory and find latest IterationInfo
    largestElastixLevel = -1
    largestResolutionLevel = -1
    latestFile = ""
    for subdir in os.walk( options.directory ):
        for i in subdir[2]:
            index = i.find( "IterationInfo" )
            if index != -1:
                fileNameParts = i.split( "." )
                currentElastixLevel = int( fileNameParts[1] )
                currentResolutionLevel = int( fileNameParts[2].lstrip('R') )

                if currentElastixLevel > largestElastixLevel:
                    largestElastixLevel = currentElastixLevel
                    largestResolutionLevel = currentResolutionLevel
                    latestFile = i
                elif currentElastixLevel == largestElastixLevel and currentResolutionLevel > largestResolutionLevel:
                    largestResolutionLevel = currentResolutionLevel
                    latestFile = i

    # Sanity check
    if latestFile == "":
      print( "ERROR: no IterationInfo files found in '" + options.directory + "'" )
      return 1

    print( "The latest iteration file is '" + latestFile + "'" )

    # Read last line of IterationInfo file
    fileName = options.directory + "/" + latestFile
    f = open( fileName )
    lineList = f.readlines()
    f.close()
    lastLine = lineList[ len(lineList) - 1 ]

    # Split the last line and the baseline
    lastLine = lastLine.rstrip( "\n" )
    print( "The final registration result has values:\n'" + lastLine + "'" )
    print( "The baseline values are:\n'" + options.baseline + "'" )

    lastLineValues = lastLine.replace('\t', ' ').replace('\n', ' ').replace('\r', ' ')
    lastLineValues = lastLineValues.split(' ')

    baselineValues = options.baseline.replace('\t', ' ').replace('\n', ' ').replace('\r', ' ')
    baselineValues = baselineValues.split(' ')


    index = 0
    for i in baselineValues:
        if i == 'x':
            index = index + 1
            continue

        fValueBaseline = float( i )
        fValueLastLine = float( lastLineValues[ index ] )

        if fValueBaseline != fValueLastLine:
            print( "ERROR: These lines are NOT the same" )
            return 1;

        index = index + 1

    print( "SUCCESS: These lines are the same" )
    return 0

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    sys.exit(main())



import sys, subprocess
import os
import os.path
import re
import shutil
import math
#import numpy
from optparse import OptionParser

#-------------------------------------------------------------------------------
# the main function
def main():
  # usage, parse parameters
  usage = "usage: %prog [options] arg";
  parser = OptionParser( usage );

  # option to debug and verbose
  parser.add_option( "-v", "--verbose", action="store_true", dest="verbose" );

  # options to control files
  parser.add_option( "-d", "--directory", dest="directory", help="elastix output directory" );
  parser.add_option( "-f", "--fixedlandmarks", dest="flm", help="fixed image landmarks" );
  parser.add_option( "-b", "--baselinetp", dest="btp", help="baseline transform parameter file" );
  parser.add_option( "-t", "--threshold", dest="threshold", help="threshold on landmark error" );
  parser.add_option( "-p", "--path", dest="path", help="path where executables can be found" );

  (options, args) = parser.parse_args();

  # Check if option -d and -f and -b are given
  if options.directory == None :
    parser.error( "The option directory (-d) should be given" );
  if options.flm == None :
    parser.error( "The option directory (-f) should be given" );
  if options.btp == None :
    parser.error( "The option directory (-b) should be given" );
  if options.threshold == None : threshold = 0.5;
  else :                         threshold = options.threshold;

  # Get the transform parameters files
  tpFileName   = os.path.join( options.directory, "TransformParameters.0.txt" );
  tpFileName_b = options.btp;

  # Sanity checks
  if not os.path.exists( tpFileName ) :
    print( "ERROR: the file " + tpFileName + " does not exist" );
    return 1;

  # Below we use programs that are compiled with elastix, and are thus available
  # in the binary directory. The user of this script has to supply the path
  # to the binary directory via the command line.
  # In order to make sure that python is able to find these programs we add
  # the paths to the local environment.
  _path = os.path.dirname( options.path );
  _path += os.pathsep + os.getenv('PATH');
  os.environ['PATH'] = _path;

  # output file of the transformix runs; and copies, for later debugging.
  landmarkstemp = os.path.join( options.directory, "outputpoints.txt" );
  landmarks1full = os.path.join( options.directory, "outputpoints_current.txt" );
  landmarks2full = os.path.join( options.directory, "outputpoints_baseline.txt" );

  # Remove copies otherwise os.rename will not work on Windows:
  # "On Windows, if dst already exists, OSError will be raised"
  if( os.path.exists( landmarks1full ) ) : os.remove( landmarks1full );
  if( os.path.exists( landmarks2full ) ) : os.remove( landmarks2full );

  #
  # Transform the fixed image landmarks by the current result
  #
  print( "Transforming fixed image landmarks using " + tpFileName );
  landmarks1 = os.path.join( options.directory, "landmarks_current.txt" );
  subprocess.call( [ "transformix", "-def", options.flm, "-out", options.directory, "-tp", tpFileName ],
    stdout=subprocess.PIPE );

  # Parse file to extract only the column with the output points
  f1 = open( landmarkstemp, 'r' );
  f2 = open( landmarks1, 'w' );
  for line in f1 :
    f2.write( line.strip().split(';')[4].strip().strip( "OutputPoint = [ " ).rstrip( " ]" ) + "\n" );
  f1.close(); f2.close();
  shutil.move( landmarkstemp, landmarks1full ); # for later inspection

  #
  # Transform the fixed image landmarks by the baseline result
  #
  print( "Transforming fixed image landmarks using " + tpFileName_b );
  landmarks2 = os.path.join( options.directory, "landmarks_baseline.txt" );
  subprocess.call( [ "transformix", "-def", options.flm, "-out", options.directory, "-tp", tpFileName_b ],
    stdout=subprocess.PIPE );
  # shutil.copyfile( landmarkstemp, landmarks2 ); // this should not be necessary

  # Parse file to extract only the column with the output points
  f1 = open( landmarkstemp, 'r' );
  f2 = open( landmarks2, 'w' );
  for line in f1 :
    f2.write( line.strip().split(';')[4].strip().strip( "OutputPoint = [ " ).rstrip( " ]" ) + "\n" );
  f1.close(); f2.close();
  shutil.move( landmarkstemp, landmarks2full ); # for later inspection

  # Compute the distance between all transformed landmarks
  f1 = open( landmarks1, 'r' ); f2 = open( landmarks2, 'r' );
  distances = [];
  for line1, line2 in zip( f1, f2 ) :
    floats1 = [ float(x) for x in line1.split() ];
    floats2 = [ float(x) for x in line2.split() ];
    diffSquared = [ (m - n) * (m - n) for m, n in zip( floats1, floats2 ) ];
    distance = math.sqrt( sum( diffSquared ) );
    distances.append( distance );

  # Compute some statistics on the distances
  distances.sort();
  minDistance  = "{0:.3f}".format( distances[ 0 ] );
  Q1           = "{0:.3f}".format( distances[ int( len( distances ) * 1.0 / 4.0 ) ] );
  medDistance  = "{0:.3f}".format( distances[ int( len( distances ) * 2.0 / 4.0 ) ] );
  Q3           = "{0:.3f}".format( distances[ int( len( distances ) * 3.0 / 4.0 ) ] );
  maxDistance  = "{0:.3f}".format( distances[ -1 ] );
  meanDistance = "{0:.3f}".format( sum( distances ) / float( len( distances ) ) );

  # With numpy it would be:
  #l1 = numpy.loadtxt( landmarks1 );
  #l2 = numpy.loadtxt( landmarks2 );
  #meandistance = numpy.mean( numpy.sum( (l1-l2)**2, axis=-1)**0.5 );

  # Report
  print( "The landmark distance between current and baseline is:" );
  print( "min   | Q1    | med   | Q3    | max   | mean" );
  print( minDistance + " | " +  Q1 + " | " +  medDistance + " | " +  Q3 + " | " + maxDistance + " | " +  meanDistance  );
  if float( Q3 ) < float( threshold ) :
    print( "SUCCESS: third quartile landmark distance is lower than " + str( threshold ) + " mm" );
    return 0;
  else :
    print( "FAILURE: third quartile landmark distance is higher than " + str( threshold ) + " mm" );
    return 1;

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    sys.exit(main())
